@@ -812,8 +812,8 @@ def d_traverse(
     ):
     """Traverse dict or list and apply callback function.
     callback function will be called as follows.
-      `callback(obj, key, value, parents=parents, *args,  **kwargs)`
-      `callback(obj, index, value, parents=parents, *args,  **kwargs)`
+      `callback(obj, key, value, *args, parents=parents, **kwargs)`
+      `callback(obj, index, value, *args, parents=parents, **kwargs)`
     """
 
     def _traverse_dict(
@@ -846,8 +846,8 @@ def d_traverse(
             parents.pop()
 
     if not _type.is_function(callback):
-            raise ValueError("callback argument must be a callable.")
-    if _type.is_mapping(obj):
+        raise ValueError("callback argument must be a callable.")
+    if _type.is_dict(obj):
         _traverse_dict(obj, callback, *args, parents=parents, **kwargs)
     elif _type.is_list_or_tuple(obj):
         _traverse_list(obj, callback, *args, parents=parents, **kwargs)
