@@ -20,10 +20,20 @@ class CloudpickleSerializer(AbstractSerializer):
                          overwrite=True)
 
     def loads(self, s, **kwargs):
+        kwargs, serializer, sunformat, encoding, options = self.parse_kwargs(**kwargs)
+        if _type.is_bytes(s):
+                s = s.decode('utf-8')
+
+        kwargs.update(options)
         data = cloudpickle.loads(s, **kwargs)
         return data
 
     def dumps(self, d, **kwargs):
+        kwargs, serializer, sunformat, encoding, options = self.parse_kwargs(**kwargs)
+        if _type.is_bytes(s):
+                s = s.decode('utf-8')
+
+        kwargs.update(options)
         data = cloudpickle.dumps(d, **kwargs)
         return data
 
