@@ -208,15 +208,15 @@ def register_format(
     # Here we generate dumper/dumpser if they are not present.
     if dumper and not dumpser:
 
-        def dumpser(obj):
+        def dumpser(obj, **kwargs):
             buf = BytesIO()
-            dumper(obj, buf)
+            dumper(obj, buf, **kwargs)
             return buf.getvalue()
 
     elif not dumper and dumpser:
 
-        def dumper(obj, fp):
-            fp.write(dumpser(obj))
+        def dumper(obj, fp, **kwargs):
+            fp.write(dumpser(obj, **kwargs))
 
     elif not dumper and not dumpser:
 
@@ -228,13 +228,13 @@ def register_format(
     # Here we generate loader/loadser if they are not present.
     if loader and not loadser:
 
-        def loadser(serialized):
-            return loader(BytesIO(serialized))
+        def loadser(serialized, **kwargs):
+            return loader(BytesIO(serialized), **kwargs)
 
     elif not loader and loadser:
 
-        def loader(fp):
-            return loadser(fp.read())
+        def loader(fp, **kwargs):
+            return loadser(fp.read(), **kwargs)
 
     elif not loader and not loadser:
 
@@ -467,7 +467,7 @@ def read_contents(s,
     if is_data(s): # data
         return s
     elif is_url(s): # url
-        return read_url(s)
+        return read_url(s, serialize=serialize)
     elif is_dsn(s): # databse
         return list(read_database(s, **kwargs))
     elif validate_file(s, thrown_error=thrown_error):
@@ -478,6 +478,8 @@ def read_contents(s,
 
 def read_url(
         url: str,
+        encoding: str='utf-8',
+        serialize: bool=False,
         **options: Any
     ):
     if not requests_installed:
@@ -486,6 +488,14 @@ def read_url(
     response = requests.get(url, **options)
     response.raise_for_status()
     contents = response.text
+    if serialize:
+        try:
+            format = pathlib.Path(url).suffix.lstrip('.')
+            if isinstance(contents, str):
+                contents = contents.encode(encoding)
+            contents = loads(contents, format)
+        except:
+            contents = response.text
     return contents
 
 def read_database(
